@@ -22,6 +22,93 @@ def export_to_json(data, file_path):
         logging.info("Data exported successfully.")
 
 
+# def fix_invalid_keys(data):
+#     """Fixes invalid keys in the data by:
+#         - Replacing periods in the keys with underscores.
+#         - Replacing the spaces in the keys with underscores.
+#         - Replacing the slashes in the keys with bars.
+#         - Replacing the at signs with hashtags.
+#
+#     Args:
+#         data (dict, list[dict]): The data to fix the keys for.
+#
+#     Returns:
+#         dict, list[dict]: The data with the invalid keys replaced.
+#
+#     Raises:
+#         TypeError: If the data is not a dictionary or a list of dictionaries.
+#     """
+    # # replace periods, spaces, and slashes in the keys
+    # if isinstance(data, dict):
+    #     clean_data = {}  # new dictionary to store the data with at signs replaced by underscores
+    #     for key, value in data.items():
+    #         if not key:  # if the key is empty, replace it with an underscore
+    #             key = '_'
+    #         if '@' in key:  # if the key has an at sign, replace it with a hashtag
+    #             key = key.replace('@', '#')
+    #         if '.' in key:  # if the key has a period, replace it with an underscore
+    #             key = key.replace('.', '_')
+    #         if ' ' in key:
+    #             key = key.replace(' ', '_')
+    #         if '/' in key:
+    #             key = key.replace('/', '|')
+    #
+    #         if isinstance(value, (dict, list)):
+    #             clean_data[key] = fix_invalid_keys(value)
+    #         else:
+    #             clean_data[key] = value
+    #
+    #     return clean_data
+    #
+    # elif isinstance(data, list):
+    #     clean_data = []
+    #     for item in data:
+    #         if isinstance(item, dict):
+    #             clean_data.append(fix_invalid_keys(item))
+    #         else:
+    #             clean_data.append(item)
+    #
+    #     return clean_data
+    #
+    # else:
+    #     raise TypeError("Data must be a dictionary or a list of dictionaries.")
+def replace_invalid_keys(data):
+    """Replaces invalid characters in the keys of the given data dict or list of dicts.
+
+    Args:
+        data (dict, list[dict]): The data to convert.
+
+    Returns:
+        dict, list[dict]: The data with invalid characters replaced in the keys.
+
+    Raises:
+        TypeError: If the data is not a dictionary or a list of dictionaries.
+    """
+    if isinstance(data, dict):
+        new_data = {}
+        for key, value in data.items():
+            new_key = key.replace('.', '_').replace(' ', '_').replace('/', '|').replace('@', '#')
+            if isinstance(value, (dict, list)):
+                new_data[new_key] = replace_invalid_keys(value)
+            else:
+                new_data[new_key] = value
+
+        return new_data
+
+    elif isinstance(data, list):
+        new_data = []
+        for item in data:
+            if isinstance(item, (dict, list)):
+                new_data.append(replace_invalid_keys(item))
+            else:
+                new_data.append(item)
+
+        return new_data
+
+    else:
+        raise TypeError("Data must be a dictionary or a list of dictionaries.")
+
+
 def initialize_logger(file_handler_path=None):
     """Initializes a logger with a stream handler and an optional file handler.
 
@@ -65,11 +152,8 @@ def initialize_logger(file_handler_path=None):
 
 def normalize_data_for_bigquery(data):
     """Normalizes the data for BigQuery by:
-    - Replacing periods in the keys with underscores.
-    - Replacing the spaces in the keys with underscores.
-    - Replacing the slashes in the keys with bars.
-    - Replacing the at signs with hashtags.
-    - Converting the data to newline delimited json.
+        - fixing invalid keys in the data.
+        - Converting the data to newline delimited json.
 
     Args:
         data (dict, list[dict]): The data to normalize.
@@ -80,14 +164,13 @@ def normalize_data_for_bigquery(data):
     Raises:
         TypeError: If the data is not a dictionary or a list of dictionaries.
     """
-    # replace periods, spaces, and slashes in the keys
-    data = replace_at_sign_in_keys(data)
-    data = replace_periods_in_keys(data)
-    data = replace_spaces_in_keys(data)
-    data = replace_slashes_in_keys(data)
+    logging.info("Normalizing data for BigQuery...")
+
+    # fix invalid keys in the data
+    clean_data = replace_invalid_keys(data)
 
     # convert to newline delimited json
-    bq_data = parse_to_nldjson(data, upload_date=True)
+    bq_data = parse_to_nldjson(clean_data, upload_date=True)
 
     return bq_data
 
@@ -127,171 +210,3 @@ def parse_to_nldjson(data_to_parse, upload_date=True):
             nld_json += f"{json.dumps(item)}\n"
 
     return nld_json
-
-
-def replace_at_sign_in_keys(data):
-    """Recursively replaces at signs in the keys of the given data dict or list of dicts with hashtags.
-
-    Args:
-        data (dict, list[dict]): The data to convert.
-
-    Returns:
-        dict, list[dict]: The data with at signs replaced by underscores in the keys.
-
-    Raises:
-        TypeError: If the data is not a dictionary or a list of dictionaries.
-    """
-    # check if the data is valid
-    if isinstance(data, dict):
-        new_data = {}  # new dictionary to store the data with at signs replaced by underscores
-        logging.info("Converting at signs to underscores in dictionary keys...")
-        for key, value in data.items():
-            new_key = key.replace('@', '#')
-            if isinstance(value, (dict, list)):
-                new_data[new_key] = replace_at_sign_in_keys(value)
-            else:
-                new_data[new_key] = value
-
-        logging.info("Dictionary keys converted successfully.")
-        return new_data
-
-    elif isinstance(data, list):  # if the data is a list
-        new_data = []
-        logging.info("Converting at signs to underscores in list of dictionary keys...")
-        for item in data:
-            if isinstance(item, (dict, list)):
-                new_data.append(replace_at_sign_in_keys(item))
-            else:
-                new_data.append(item)
-
-        logging.info("List of dictionary keys converted successfully.")
-        return new_data
-
-    else:
-        raise TypeError("Data must be a dictionary or a list of dictionaries.")
-
-
-def replace_periods_in_keys(data):
-    """Recursively replaces periods in the keys of the given data dict or list of dicts with underscores.
-
-    Args:
-        data (dict, list[dict]): The data to convert.
-
-    Returns:
-        dict, list[dict]: The data with periods replaced by underscores in the keys.
-
-    Raises:
-        TypeError: If the data is not a dictionary or a list of dictionaries.
-    """
-    # check if the data is valid
-    if isinstance(data, dict):
-        new_data = {}  # new dictionary to store the data with periods replaced by underscores
-        logging.info("Converting periods to underscores in dictionary keys...")
-        for key, value in data.items():
-            new_key = key.replace('.', '_')
-            if isinstance(value, (dict, list)):
-                new_data[new_key] = replace_periods_in_keys(value)
-            else:
-                new_data[new_key] = value
-
-        logging.info("Dictionary keys converted successfully.")
-        return new_data
-
-    elif isinstance(data, list):  # if the data is a list
-        new_data = []
-        logging.info("Converting periods to underscores in list of dictionary keys...")
-        for item in data:
-            if isinstance(item, (dict, list)):
-                new_data.append(replace_periods_in_keys(item))
-            else:
-                new_data.append(item)
-
-        logging.info("List of dictionary keys converted successfully.")
-        return new_data
-
-    else:
-        raise TypeError("Data must be a dictionary or a list of dictionaries.")
-
-
-def replace_slashes_in_keys(data):
-    """Recursively replaces slashes in the keys of the given data dict or list of dicts with bars.
-
-    Args:
-        data (dict, list[dict]): The data to convert.
-
-    Returns:
-        dict, list[dict]: The data with slashes replaced by bars in the keys.
-
-    Raises:
-        TypeError: If the data is not a dictionary or a list of dictionaries.
-    """
-    # check if the data is valid
-    if isinstance(data, dict):
-        new_data = {}  # new dictionary to store the data with slashes replaced by bars
-        logging.info("Converting slashes to bars in dictionary keys...")
-        for key, value in data.items():
-            new_key = key.replace('/', '|')
-            if isinstance(value, (dict, list)):
-                new_data[new_key] = replace_slashes_in_keys(value)
-            else:
-                new_data[new_key] = value
-
-        logging.info("Dictionary keys converted successfully.")
-        return new_data
-
-    elif isinstance(data, list):  # if the data is a list
-        new_data = []
-        logging.info("Converting slashes to bars in list of dictionary keys...")
-        for item in data:
-            if isinstance(item, (dict, list)):
-                new_data.append(replace_slashes_in_keys(item))
-            else:
-                new_data.append(item)
-
-        logging.info("List of dictionary keys converted successfully.")
-        return new_data
-
-    else:
-        raise TypeError("Data must be a dictionary or a list of dictionaries.")
-
-
-def replace_spaces_in_keys(data):
-    """Recursively replaces spaces in the keys of the given data dict or list of dicts with underscores.
-
-    Args:
-        data (dict, list[dict]): The data to convert.
-
-    Returns:
-        dict, list[dict]: The data with spaces replaced by underscores in the keys.
-
-    Raises:
-        TypeError: If the data is not a dictionary or a list of dictionaries.
-    """
-    # check if the data is valid
-    if isinstance(data, dict):
-        new_data = {}  # new dictionary to store the data with spaces replaced by underscores
-        logging.info("Converting spaces to underscores in dictionary keys...")
-        for key, value in data.items():
-            new_key = key.replace(' ', '_')
-            if isinstance(value, (dict, list)):
-                new_data[new_key] = replace_spaces_in_keys(value)
-            else:
-                new_data[new_key] = value
-
-        logging.info("Dictionary keys converted successfully.")
-        return new_data
-
-    elif isinstance(data, list):  # if the data is a list
-        new_data = []
-        logging.info("Converting spaces to underscores in list of dictionary keys...")
-        for item in data:
-            if isinstance(item, (dict, list)):
-                new_data.append(replace_spaces_in_keys(item))
-            else:
-                new_data.append(item)
-
-        logging.info("List of dictionary keys converted successfully.")
-        return new_data
-
-    else:
-        raise TypeError("Data must be a dictionary or a list of dictionaries.")
